@@ -22,9 +22,11 @@ __docformat__ = "restructuredtext en"
 import warnings
 from datetime import datetime, date, time
 
+from six import string_types, text_type
+from six.moves import builtins
+
 # XXX set _ builtin to unicode by default, should be overriden if necessary
-import __builtin__
-__builtin__._ = unicode
+builtins._ = text_type
 
 from logilab.common.date import strptime, strptime_time
 from logilab.common import nullobject
@@ -42,6 +44,21 @@ BASE_TYPES = set(('String', 'Password', 'Bytes',
 # base groups used in permissions
 BASE_GROUPS = set((_('managers'), _('users'), _('guests'), _('owners')))
 
+# default permissions for relations and attributes
+DEFAULT_RELPERMS = {'read': ('managers', 'users', 'guests',),
+                    'delete': ('managers', 'users'),
+                    'add': ('managers', 'users',)}
+DEFAULT_ATTRPERMS = {'read': ('managers', 'users', 'guests',),
+                     'add': ('managers', 'users'),
+                     'update': ('managers', 'owners')}
+DEFAULT_COMPUTED_RELPERMS = {'read': ('managers', 'users', 'guests',),
+                             'delete': (),
+                             'add': ()}
+DEFAULT_COMPUTED_ATTRPERMS = {'read': ('managers', 'users', 'guests',),
+                              'add': (),
+                              'update': ()}
+
+
 # This provides a way to specify callable objects as default values
 # First level is the final type, second is the keyword to callable map
 KEYWORD_MAP = {
@@ -64,7 +81,7 @@ DATE_FACTORY_MAP = {
 def convert_default_value(rdef, default):
     # rdef can be either a yams.schema.RelationDefinitionSchema or a yams.buildobjs.RelationDefinition
     rtype = getattr(rdef, 'name', None) or rdef.rtype.type
-    if isinstance(default, basestring) and rdef.object != 'String':
+    if isinstance(default, string_types) and rdef.object != 'String':
         # real Strings can be anything, including things that look like keywords
         # for other base types
         if rdef.object in KEYWORD_MAP:
@@ -81,12 +98,12 @@ def convert_default_value(rdef, default):
                           DeprecationWarning)
             try:
                 return DATE_FACTORY_MAP[rdef.object](default)
-            except ValueError, verr:
+            except ValueError as verr:
                 raise ValueError('creating a default value for attribute %s of type %s '
                                  'from the string %r is not supported (cause %s)'
                                  % (rtype, rdef.object, default, verr))
     if rdef.object == 'String':
-        default = unicode(default)
+        default = text_type(default)
     return default # general case: untouched default
 
 
@@ -100,7 +117,7 @@ def register_base_type(name, parameters=(), check_function=None):
     from yams.schema import RelationDefinitionSchema
     from yams.constraints import BASE_CHECKERS, yes
     # Add the datatype to yams base types
-    assert name not in BASE_TYPES, '%s alreadt in BASE_TYPES %s' % (name, BASE_TYPES)
+    assert name not in BASE_TYPES, '%s already in BASE_TYPES %s' % (name, BASE_TYPES)
     BASE_TYPES.add(name)
     # Add the new datatype to the authorized types of RelationDefinitionSchema
     if not isinstance(parameters, dict):
