@@ -24,7 +24,7 @@ __docformat__ = "restructuredtext en"
 modname = 'yams'
 
 # release version
-numversion = (0, 39, 1)
+numversion = (0, 40, 0)
 version = '.'.join(str(num) for num in numversion)
 
 # license and copyright
@@ -43,16 +43,13 @@ author = "Logilab"
 author_email = "devel@logilab.fr"
 
 # home page
-web = "http://www.logilab.org/project/%s" % modname
+web = "https://www.logilab.org/project/%s" % modname
 
 # mailing list
 mailinglist = 'mailto://python-projects@lists.logilab.org'
 
 # download place
-ftp = "ftp://ftp.logilab.org/pub/%s" % modname
-
-# is there some directories to include with the source installation
-include_dirs = []
+ftp = "http://download.logilab.org/pub/%s" % modname
 
 # executable
 
@@ -61,4 +58,5 @@ scripts = ['bin/yams-check', 'bin/yams-view']
 install_requires = [
     'logilab-common >= 0.55.2',
     'logilab-database >= 1.11',
+    'six >= 1.4.0',
     ]
