@@ -1,11 +1,11 @@
 import logging
-from cStringIO import StringIO
+from io import StringIO
 from logilab.common.graph import ordered_nodes
 
 def serialize_to_python(s):
     out = StringIO()
     w = out.write
-    w('from yams.buildobjs import *\n\n')
+    w(u'from yams.buildobjs import *\n\n')
     graph = {}
     for entity in s.entities():
         l = graph.setdefault(entity, [])
@@ -17,21 +17,21 @@ def serialize_to_python(s):
                 base = e._specialized_type
             else:
                 base = 'EntityType'
-            w('class %s(%s):\n' % (e.type, base))
+            w(u'class %s(%s):\n' % (e.type, base))
             attr_defs = list(e.attribute_definitions())
             if attr_defs:
                 for attr,obj in attr_defs:
-                    w('    %s = %s()\n' % (attr.type, obj.type))
+                    w(u'    %s = %s()\n' % (attr.type, obj.type))
             else:
-                w('    pass\n')
-            w('\n')
+                w(u'    pass\n')
+            w(u'\n')
     for r in s.relations():
         if not r.final:
             if r.subjects() and r.objects():
-                w('class %s(RelationDefinition):\n' % r.type)
-                w('    subject = (%s,)\n' % ', '.join("'%s'" % x for x in r.subjects()))
-                w('    object = (%s,)\n' % ', '.join("'%s'" % x for x in r.objects()))
-                w('\n')
+                w(u'class %s(RelationDefinition):\n' % r.type)
+                w(u'    subject = (%s,)\n' % ', '.join("'%s'" % x for x in r.subjects()))
+                w(u'    object = (%s,)\n' % ', '.join("'%s'" % x for x in r.objects()))
+                w(u'\n')
             else:
                 logging.warning('relation definition %s missing subject/object' % r.type)
     return out.getvalue()
